from glob_utils import APIClient
from pydantic import Field
from os import environ
from pinedantic._types import QueryRequest, QueryResponse, UpsertRequest, UpsertResponse, QueryMatch, UpsertVectors, Vector, Filter

class Pinecone(APIClient):
	base_url:str = Field(default=environ.get('PINECONE_URL', 'https://api.pinecone.io'))
	headers:dict[str,str] = Field(default_factory=lambda: {'api-key': environ.get('PINECONE_API_KEY')})

	async def query(self, vector:Vector, filter:Filter, topK:int=10) -> QueryResponse:
		request = QueryRequest(vector=vector, filter=filter, topK=topK)
		response = await self.post('/query', json=request.dict())
		return QueryResponse(**response)
	
	async def upsert(self, vectors:list[Vector]) -> UpsertResponse:
		request = UpsertRequest(vectors=vectors)
		response = await self.post('/vectors/upsert', json=request.dict())
		return UpsertResponse(**response)


from __future__ import annotations
from typing import TypeAlias, Literal, Union
from pydantic import BaseModel, Field
from uuid import uuid4

Vector: TypeAlias = list[float]
RelationalOperator: TypeAlias = Literal["$eq", "$gt", "$gte", "$in", "$lt", "$lte", "$ne", "$nin"]
LogicalOperator: TypeAlias = Literal["$and", "$or"]
Value: TypeAlias = Union[str, int, float, bool, list[str]]
Filter: TypeAlias = Union[dict[str,dict[RelationalOperator,Value]],dict[LogicalOperator,list[dict[str,dict[RelationalOperator,Value]]]]] 

class QueryRequest(BaseModel):
	vector: Vector
	includeMetadata: bool = Field(default=True)
	filter: Filter = Field(default_factory=dict)
	topK: int = Field(default=10)

class QueryMatch(BaseModel):
	id: str
	score: float
	metadata: dict[str,Value]

class QueryResponse(BaseModel):
	matches: list[QueryMatch]

class UpsertVectors(BaseModel):
	id: str = Field(default_factory=lambda: str(uuid4()))
	values: list[Vector]
	metadata: dict[str,Value]

class UpsertRequest(BaseModel):
	vectors: list[UpsertVectors]

class UpsertResponse(BaseModel):
	upsertedCount: int